#!/usr/bin/perl
#
#
# Copyright (c) 2012 Peter Varkoly <peter@varkoly.de> Nürnberg, Germany.  All rights reserved.
# Copyright (c) 2009 Peter Varkoly <peter@varkoly.de>, Fürth.  All rights reserved.
#
# $Id: oss-client-control.pl pv Exp $
#
#
=head1 NAME

 oss-client-control-server-daemon.pl
 
=head1 PREFACE

 Control daemon for OSS Clients on the server side.

=head1 DESCRIPTION

This daemon is waiting for messages from the Clients of the OSS.


=cut

$| = 1; # do not buffer stdout

use strict;
use Getopt::Long;
use IO::Socket;
use IO::Socket::SSL;
use Data::Dumper;

$IO::Socket::SSL::DEBUG=1;

my $LocalPort = '1969';
my $LocalAddr = 'admin';
my ($client, $client_address);

#Global Variables
my %REPLY   = ();
my $CDATE   = undef;
my $COMMAND = undef;
my $TAG     = undef;
my $TEXT    = undef;
my $OS      = 'Linux';
my $LOGDIR  = "/var/log/";
my $PIDFILE = "/var/run/oss-client-control-server-daemon.pid";
my $DEBUG   = 0;
#parsing parameter
my %options = ();
my $fhostname = `hostname -f`;
chomp $fhostname;
my( $hostname, $domainname ) = split/\./,$fhostname,2;

open FILE,">$PIDFILE";
print FILE $$;
close FILE;

sub xml_time()
{
    my  $timezone = `/bin/date +%:z`; chomp $timezone;
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    return sprintf('%4d-%02d-%02dT%02d:%02d:%02d.000%s', $year+1900,$mon+1,$mday,$hour,$min,$sec,$timezone);
}


#start the socket
my $server = IO::Socket::SSL->new( 
	Listen		=> 1,
	LocalAddr       => $LocalAddr,
	LocalPort       => $LocalPort,
	Proto	        => 'tcp',
	Reuse		=> 1,
	SSL_key_file    => "/etc/ssl/servercerts/certs/$fhostname.key.pem",
	SSL_cert_file   => "/etc/ssl/servercerts/certs/$fhostname.cert.pem",
	SSL_ca_file     => '/etc/ssl/servercerts/cacert.pem',
	SSL_use_cert    => 0,
	SSL_verify_mode => 0x01,
	Type            => SOCK_STREAM
);

if( ! defined $server ) {
    print STDERR "ERRNO=<$!> in my_connect\n";
}


$server->autoflush(1);
print "Socket started\n";

while( $client = $server->accept()  ) 
{
	my $ip    = $client->peerhost();
	my $name  = `oss_api_text.sh GET devices/hostnameByIP/$ip`;
	if( !$name ) {
		print "Can not get the hostname of device with IP $ip\n";
		return;
	}
	print " $name $ip started\n";
	my $token = `/usr/bin/uuidgen`;
	$token    =~ s/-//g;
	$token    = substr($token,0,28);
	syswrite $client,$token,28;
	open OUT, ">/var/adm/oss/toke-$name";
	print OUT $token;
	close OUT;
	system("/usr/sbin/oss-install-oss-client.pl $name &");
}


