#!/usr/bin/perl

BEGIN{ push @INC,"/usr/share/oss/lib/"; }

use strict;
use oss_base;
use oss_utils;
use Getopt::Long;

my $SYNCDESKTOP = 0;
my %options     = ();
my $result      = GetOptions(\%options,"syncdesktop","help","default-hwconf=s");
my $DEFHWCONF   = "Win10-64-Domain";

if( defined($options{'help'}) )
{
	print "\n";
	print "export-for-4-0.pl [<options>]\n";
        print "Options:\n";
        print "  --help            Print this help message\n";
        print "  --syncdesktop     Synchronize the windows profile desktop into home WinDesktop.\n";
        print "  --default-hwconf  Set this as hwconf for rooms and devices which does not have hwconfs.\n";
	print "                    This must be exist. Default is Win10-64-Domain\n\n";
        exit 0;
}

if( defined($options{'syncdesktop'}) )
{
	$SYNCDESKTOP = 1;
}

if( defined($options{'default-hwconf'}) )
{
	$DEFHWCONF = $options{'default-hwconf'};
}

my $oss = oss_base->new();

my @rooms   = $oss->get_rooms('all');
my @hwconfs = @{$oss->get_HW_configurations(0)};

my $users  = {};
my $uids   = {};

my $nm_to_count = {
	19 => 8192,
	20 => 4096,
	21 => 2048,
	22 => 1024,
	23 => 512,
	24 => 256,
	25 => 128,
	26 => 64,
	27 => 32,
	28 => 16,
	29 => 8,
	30 => 4,
	31 => 2
};

my $nm_to_places = {
	19 => 91,
	20 => 64,
	21 => 46,
	22 => 33,
	23 => 23,
	24 => 16,
	25 => 12,
	26 => 8,
	27 => 6,
	28 => 4,
	29 => 3,
	30 => 2,
	31 => 2
};

my $controlMap = {
        'in_room_control'     => 'inRoom',
        'no_control'          => 'no',
        'all_teacher_control' => 'allTeachers',
        'teacher_control'     => 'teachers'
};

my $DEVICES="room;name;mac;ip;wlanmac;wlanip;hwconf;row;place;serial;inventary;locality;owner\n";
my $ROOMS="name;description;hwconf;startip;count;rows;places;control;type\n";
my $DPFILE="";

foreach my $room ( @rooms )
{
	next if ( $room->[1] eq 'ANON_DHCP' );
	#next if ( $room->[1] eq 'SERVER_NET' );
	my $roomdn    = $room->[0];
	my $room_name = lc($room->[1]);
	my $room_hw_conf  = get_hw_conf($oss->get_config_value($roomdn,'HW'));
	my $room_start_ip = $oss->get_attribute($roomdn,'dhcpRange');
	my $room_count    = $nm_to_count->{$oss->get_attribute($roomdn,'dhcpNetMask')};
	my $places        = $nm_to_places->{$oss->get_attribute($roomdn,'dhcpNetMask')};
	my ( $control, $controller, $controllers )  = $oss->get_room_control_state($roomdn );
	my $room_control = $controlMap->{$control};
	my $room_type = 'ComputerRoom';
	if( $oss->get_config_value($roomdn,'AdHocWlanAccess') eq "yes" ) {
		$room_type = 'AdHocAccess';
	}

	my $x = $places;
	my $y = $places;
	my $xy = $oss->get_vendor_object($roomdn,'EXTIS','COORDINATES');
	if( defined $xy->[0] )
	{
		( $x,$y ) = split /,/ , $xy->[0];
		$x = "" if( $x < 1 );
		$y = "" if( $y < 1 );
	}
	my $DP = $oss->get_vendor_object($roomdn,'EXTIS','DEFAULT_PRINTER');
	if( defined $DP->[0] ) {
                $DPFILE.="oss_api.sh PUT rooms/text/$room_name/defaultPrinter/".$DP->[0]."\n";
        }

	if ( $room->[1] ne 'SERVER_NET' ) {
		$ROOMS.="$room_name;$room_name;$room_hw_conf;$room_start_ip;$room_count;$x;$y;$room_control;$room_type\n";
	}

	foreach my $dn ( @{$oss->get_workstations_of_room($roomdn)} )
	{
		my $pc_name = lc($oss->get_attribute($dn,'cn'));
		my $pc_hw_config = get_hw_conf($oss->get_config_value($dn,'HW'));
		my $pc_serial   = $oss->get_config_value($dn,'SERIALNUMBER');
		my $pc_inventar = $oss->get_config_value($dn,'INVENTARNUMBER');
		my $pc_locality = $oss->get_config_value($dn,'LOCALITY');
		my $pc_mac      = $oss->get_attribute($dn,"dhcpHWAddress"); $pc_mac =~ s/ethernet //;
		my $pc_ip       = $oss->get_ip_of_host($dn);
		my $wlan_mac    = "";
		my $wlan_ip     = "";
		my $wlanDN      = $oss->get_host($pc_name.'-wlan');
		if( $wlanDN ) {
			my $wlan_mac      = $oss->get_attribute($wlanDN,"dhcpHWAddress"); $wlan_mac =~ s/ethernet //;
			my $wlan_ip       = $oss->get_ip_of_host($wlanDN);
		}
		my $x = "";
		my $y = "";
		my $xy = $oss->get_vendor_object($dn,'EXTIS','COORDINATES');
		if( defined $xy->[0] )
		{
			( $x,$y ) = split /,/ , $xy->[0];
			$x = "" if( $x < 1 );
			$y = "" if( $y < 1 );
		}
		my $owner = $oss->get_config_value($dn,'DEVICE_OWNER');
		if( defined $owner ) {
		  $owner = get_name_of_dn($owner);
		  $pc_hw_config = "BYOD";
		} else {
		  $owner="";
		}
		$DEVICES.="$room_name;$pc_name;$pc_mac;$pc_ip;$wlan_mac;$wlan_ip;$pc_hw_config;$x;$y;$pc_serial;$pc_inventar;$pc_locality;$owner\n";
	}
	
}

write_file( "rooms.csv", $ROOMS );
write_file( "devices.csv", $DEVICES );
write_file( "default_printers.sh", $DPFILE );

my $result = $oss->{LDAP}->search( base   => $oss->{SYSCONFIG}->{COMPUTERS_BASE},
                                       scope   => 'one',
                                       filter  => '(Objectclass=SchoolConfiguration)'
                          );
if($result->code)
{
    $oss->ldap_error($result);
    print LOG "Error by reading the HWConfs.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by reading the HWConfs.\n");
}

my @HWCONFS = ();
foreach my $entry ( $result->entries )
{
    my $cn = $entry->get_value("configurationKey");
    my $description = $entry->get_value("description");
    my $name = $description;
    $name =~ s/  / /g;
    $name =~ s/ /-/g;
    $name = substr($name,0,31);
    my @values  = $entry->get_value("configurationValue");
    my $WSType  = "FatClient";
    my $part    = {};
    foreach(@values){
        my($key,$value) = split /=/,$_;
        $WSType = $value if( $key eq "WSType");
        if( $key =~ /PART_(.*)_(.*)/ ) {
            $part->{$1}->{$2} = $value;
        }
    }
    my $HWCONF .= "{\"name\":\"$name\",\"description\":\"$description\",\"deviceType\":\"$WSType\",\"partitions\":[";
    my @Partitions = ();
    foreach(keys %$part) {
        my $desc   = substr($part->{$_}->{DESC},0,30);
	$desc =~ s/\t/ /g;
        my $JOIN   = defined $part->{$_}->{JOIN}   ? $part->{$_}->{JOIN}   : "";
        my $OS     = defined $part->{$_}->{OS}     ? $part->{$_}->{OS}     : "";
        my $FORMAT = defined $part->{$_}->{FORMAT} ? $part->{$_}->{FORMAT} : "";
        my $ITOOL  = defined $part->{$_}->{ITOOL}  ? $part->{$_}->{ITOOL}  : "";
        if( $JOIN eq "Simple" ) {
                $JOIN = "Domain";
        }
	push @Partitions, "{\"name\":\"$_\",\"description\":\"$desc\",\"os\":\"$OS\",\"joinType\":\"$JOIN\",\"tool\":\"$ITOOL\",\"format\":\"$FORMAT\"}";
    }
    $HWCONF .= join(",",@Partitions)."]}";
    push @HWCONFS, $HWCONF;
}

write_file( "hwconfs.csv", '['.join(',',@HWCONFS).']');

my $result = $oss->{LDAP}->search(  base   => $oss->{SYSCONFIG}->{USER_BASE},
                                  scope  => "one",
                                  filter => "objectclass=*",
                                  attrs  => [ 'uid','uniqueidentifier','sn','givenName','role','birthDay','susemailacceptaddress','susemailforwardaddress','uidNumber','homeDirectory' ]
                                );

open USERS, ">:encoding(UTF-8)","chown-users.sh";
foreach my $entry ( $result->entries )
{
    my $uid = lc($entry->get_value("uid"));
    $uids->{$uid} = 1;
    next if( $uid eq "admin" );
    my $sn  = $entry->get_value("sn");
    my $givenName = $entry->get_value("givenName");
    my $uidNumber = $entry->get_value("uidNumber");
    my $birthDay = $entry->get_value("birthDay");
    my $uuid = $entry->get_value("uniqueidentifier") || '';
    my $role = $entry->get_value("role"); $role =~ s/,sysadmins//;
    next if ( !defined $role );
    next if ( $role eq "workstations" );
    next if ( $role eq "templates" );
    my $uidNumber     = $entry->get_value("uidNumber");
    my $homeDirectory = $entry->get_value("homeDirectory");
    my @suseMailAcceptAddress = $entry->get_value("suseMailAcceptAddress");
    my @suseMailForwardAddess = $entry->get_value("suseMailForwardAddess");
    if( $SYNCDESKTOP ) {
	print("Syncing /home/profile/$uid/*/Desktop/ $homeDirectory/WinDesktop/\n");
	system("rsync -a /home/profile/$uid/*/Desktop/ $homeDirectory/WinDesktop/");
    }
    my @classess = ();
    for my $class ( @{ $oss->get_classes_of_user($entry->dn())} ) {
        push @classess, get_name_of_dn($class);
    }
    Encode::_utf8_on($sn);
    Encode::_utf8_on($givenName);
    $sn =~ s/'/\\'/g;
    $givenName =~ s/'/\\'/g;
    push @{$users->{$role}}, "$uid;$uuid;$sn;$givenName;$birthDay;".join(' ',@classess);
    print USERS "if id $uid\n";
    print USERS "then\n";
    print USERS "   echo '$uid exists'\n";
    print USERS "   find /home/all/ /home/groups/ /home/software/ -uid $uidNumber -exec chown $uid {} \\;\n";
    print USERS "else\n";
    print USERS "   find /home/all/ /home/groups/ /home/software/ -type f -uid $uidNumber -exec rm -f {} \\;\n";
    print USERS "   find /home/all/ /home/groups/ /home/software/ -type d -uid $uidNumber -exec rm -rf {} \\;\n";
    print USERS "fi\n";
    print USERS "echo 'Processed $uid'\n";
}
close USERS;

foreach my $role ( keys %$users ) {
    open USERS, ">:encoding(UTF-8)","$role.csv";
    print USERS "UID;UUID;SN;GIVENNAME;BIRTHDAY;CLASS\n";
    foreach(@{$users->{$role}})
    {
        print USERS $_."\n";
    }
    close USERS;
}

$result = $oss->{LDAP}->search(  base   => $oss->{SYSCONFIG}->{GROUP_BASE},
                                  scope  => "one",
                                  filter => "objectclass=*",
                                  attrs  => [ 'cn','groupType','member','description','susemailacceptaddress','susemailforwardaddress','gidNumber','creatorsName' ]
                                );

if($result->code)
{
    $oss->ldap_error($result);
    print LOG "Error by reading the groups.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by reading the groups.\n");
}

open GROUPS, ">:encoding(UTF-8)","groups.csv";
print GROUPS "";
foreach my $entry ( $result->entries )
{
    my $cn = lc($entry->get_value("cn"));
    if( defined $uids->{$cn} ) {
        print LOG "ERROR Group $cn can not be migrated. A user with the same name does exists.\n";
        $oss->{LDAP}->delete( $entry->dn() );
        next;
    }
    next if $cn eq "administration";
    next if $cn eq "domainusers";
    next if $cn eq "guest";
    next if $cn eq "guests";
    next if $cn eq "students";
    next if $cn eq "teachers";
    next if $cn eq "templates";
    next if $cn eq "workstations";
    my $groupType   = $entry->get_value("groupType");
    next if $groupType eq "class";
    my $gidNumber   = $entry->get_value("gidNumber");
    my $description = $entry->get_value("description") || $cn;
    my $creator     = $entry->get_value("creatorsName"); $creator = lc(get_name_of_dn($creator));
    my @member  = $entry->get_value("member");
    my @tmp = ();
    foreach my $udn (@member) {
        my $uid = get_name_of_dn($udn);
        if( defined $uids->{$uid} ) {
            push @tmp,$uid if( $uid ne "Administrator" && $uid ne "admin" );
        }
    }
    print GROUPS "$cn;$description;$groupType;".join(" ",@tmp)."\n";
}
close(GROUPS);

sub get_hw_conf($)
{
	my $name = shift;
	my $hw_config_description = $DEFHWCONF;
	foreach my $hwconfig (@hwconfs)
        {
		if( $hwconfig->[0] eq $name )
		{
		        $hw_config_description = $hwconfig->[1];
		        last;
		}
        }
        $hw_config_description =~ s/  / /g;
        $hw_config_description =~ s/ /-/g;
        $hw_config_description = substr($hw_config_description,0,31);
	return $hw_config_description;
}

