#!/usr/bin/perl

BEGIN{
    push @INC,"/usr/share/oss/lib/";
}

use strict;
use oss_utils;
use oss_base;
use Config::IniFiles;
use Crypt::SmbHash;
use MIME::Base64;
use Net::Netmask;
use DBI;
use DBI qw(:utils);
use Encode;

my $oss = oss_base->new();
my $administrator = undef;
my $root          = undef;

#Read some LMD settings from /etc/sysconfig/lmd
my ( $APPS_NOT_TO_ARCHIVE, $APPS_TO_ARCHIVE, $ARCHIVE_REQUESTS, $ORDER, $DBCON, $DBUSER, $DBPW, $SAVE_PASSWORD_IN_DB, $BAD_LOGIN_TIMEOUT, $MA, $CEPHALIX_SDN ) =
        parse_file( "/etc/sysconfig/lmd",
        "LMD_APPLICATIONS_NOT_TO_ARCHIVE=",
        "LMD_APPLICATIONS_TO_ARCHIVE=",
        "LMD_ARCHIVE_REQUESTS=",
        "LMD_CATEGORY_ORDER=",
        "LMD_DB_CONNECTION=",
        "LMD_DB_USER=",
        "LMD_DB_PW=",
        "LMD_SAVE_PASSWORD_IN_DB=",
        "LMD_BAD_LOGIN_TIMEOUT=",
        "LMD_MOBILE_MODULES=",
        "LMD_CEPHALIX_SDN=");

#Make DB Connection;
if( ! $DBCON )
{
        $DBCON = 'dbi:mysql:lmd';
        $DBUSER= 'root';
        ($DBPW)= parse_file('/root/.my.cnf',"password=");
}
my $DBH = DBI->connect( $DBCON, $DBUSER, $DBPW);
$DBH->do("SET CHARACTER SET utf8");
$DBH->do("SET NAMES utf8");


sub error($) {
    print "An error is occurred during the preparation for the migration.\n";
    print "Please notice this message and contact oss-support\@extis.de.\n";
    print shift;
    exit  1;
}

my $DIR="/home/archiv/migrate-to-4-0";
my $DATE=`/usr/share/oss/tools/oss_date.sh`; chomp $DATE;
open LOG, ">/var/log/OSS-MIGRATE-TO-4-0-prepare";


my $controlMap = {
	'in_room_control'     => 'inRoom',
	'no_control'	      => 'no',
	'all_teacher_control' => 'allTeachers',
	'teacher_control'     => 'teachers' 
};

my $softwareStatusMap  = {
	'installed'                => 'I',
	'installation_scheduled'   => 'IS',
	'installation_failed'      => 'IF',
	'installed_manual'         => 'MI',
	'deinstalled_manual'       => 'MD',
	'deinstallation_scheduled' => 'DS',
	'deinstallation_failed'    => 'DF'
};

my $DOMAIN = $oss->get_school_config('SCHOOL_DOMAIN');;
#TODO make some checks
#Do everything running

###################################################################################
#Do we have enough place
my $free = `df /home/ | grep '/home' | gawk '{ print \$2/1024 }'`;
if( $free < 1024 ) {
    error "There is not enoug place for the migration.\n";
}

###################################################################################
# First we have to take care that the user administrator do exists and has SID 500
my $adminDN = 'uid=admin,'.$oss->{SYSCONFIG}->{USER_BASE};
my $sambaNTPassword = $oss->get_attribute($adminDN,'sambaNTPassword');
my $mesg = $oss->{LDAP}->search( base   => $oss->{SCHOOL_BASE},
				  filter => "(&(objectclass=sambaDomain)(sambaDomainName=".$oss->{SYSCONFIG}->{SCHOOL_WORKGROUP}."))",
				  scope  => 'sub',
				  attrs  => [ 'sambaSID','sambaAlgorithmicRidBase' ]
);
if( $mesg->code() || $mesg->count() != 1 )
{
    $mesg = $oss->{LDAP}->search( base   => $oss->{LDAP_BASE},
				  filter => "(&(objectclass=sambaDomain)(sambaDomainName=".$oss->{SYSCONFIG}->{SCHOOL_WORKGROUP}."))",
				  scope  => 'sub',
				  attrs  => [ 'sambaSID','sambaAlgorithmicRidBase' ]
    );
    if( $mesg->code() || $mesg->count() != 1 )
    {
	    error "SAMBA-DOMAIN-NOT-FOUND";
    }
}
my $sambaSID =  $mesg->entry(0)->get_value('sambaSID');

$mesg = $oss->{LDAP}->search( base   => $oss->{SYSCONFIG}->{USER_BASE},
			      scope  => 'one',
			      filter => "(uid=root)",
			      attrs  => [ 'dn' ]
			);
if( ! $mesg->code() and $mesg->count() == 1 )
{
	    $root = $mesg->entry(0)->dn();
}

$mesg = $oss->{LDAP}->search( base   => $oss->{SYSCONFIG}->{USER_BASE},
			      scope  => 'one',
			      filter => "(uid=administrator)",
			      attrs  => [ 'dn' ]
			);
if( ! $mesg->code() and $mesg->count() == 1 )
{
    $administrator = $mesg->entry(0)->dn();
}
if( defined $administrator ) {
	# Delete admin from database
	$oss->delete_ldap_children($administrator);
	$oss->{LDAP}->delete($administrator);
} elsif( defined $root ) {
	# Delete admin from database
	$oss->delete_ldap_children($root);
	$oss->{LDAP}->delete($root);
}
$mesg = $oss->{LDAP}->modify( $adminDN, replace => { sambaSID =>  "$sambaSID-500" } );
if( $mesg->code() ) {
    $oss->ldap_error($mesg);
    print LOG "Error by modifying the admins sambaSID.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by modifying the admins sambaSID.\n");
}
$mesg = $oss->{LDAP}->modify( $adminDN, replace => { displayName =>  "Domain Administrator" } );
if( $mesg->code() ) {
    $oss->ldap_error($mesg);
    print LOG "Error by modifying the admins display name.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by modifying the admins display name.\n");
}
$mesg = $oss->{LDAP}->moddn( $adminDN, newrdn => "uid=Administrator", deleteoldrdn => 1 );
if( $mesg->code() ) {
    $oss->ldap_error($mesg);
    print LOG "Error by rebasing the admin.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by rebasing the admin.\n");
}

################################################

my $cmd = "/usr/share/oss/tools/reset_password_age.pl";
################################################
# Create the neccesary directory and make backup
$cmd  = "mkdir -m 700 -p $DIR;\n";
$cmd .= "mkdir -m 700 -p $DIR/SAMBA;\n";
$cmd .= "cp /etc/samba/smb.conf $DIR/SAMBA;\n";
$cmd .= "cp /etc/samba/secrets.tdb $DIR/SAMBA;\n";
$cmd .= "rsync --exclude netlogon -av /var/lib/samba/ $DIR/SAMBA/;\n";
$cmd .= "slapcat > $DIR/SLAPCAT-$DATE;";
$cmd .= "cp /usr/share/oss/oss-migration-3-to-4/add_unix_attributes.pl $DIR/\n";

print LOG $cmd;
system( $cmd );


#################################################
# Setup with the old original password
my $ADMINPW=`oss_get_admin_pw`;
open SQL, ">:encoding(UTF-8)","$DIR/OSS-MIGRATE-TO-4-0-prepare.sql";
open CMD, ">:encoding(UTF-8)","$DIR/OSS-MIGRATE-TO-4-0.sh";
open DNS, ">:encoding(UTF-8)","$DIR/OSS-MIGRATE-TO-4-0.dns";
print DNS <<EOF;
#!/bin/bash -x
REGPW=\$1
EOF

print CMD <<EOF;
#!/bin/bash -x

# First we have to merge the configuration
sed -i s/SCHOOL_NETBIOSNAME=.*/SCHOOL_NETBIOSNAME="admin"/ /etc/sysconfig/schoolserver
/usr/share/oss/oss-migration-3-to-4/merge-config.sh
. /etc/sysconfig/schoolserver
export password='$ADMINPW'
echo "Start and setup mysql"
systemctl restart mysql
systemctl enable  mysql
sleep 3
systemctl restart mysql
SERVER_NETWORK=\$( echo \$SCHOOL_SERVER_NET | gawk -F '/' '{ print \$1 }' )
SERVER_NETMASK=\$( echo \$SCHOOL_SERVER_NET | gawk -F '/' '{ print \$2 }' )
ANON_NETWORK=\$( echo \$SCHOOL_ANON_DHCP_NET | gawk -F '/' '{ print \$1 }' )
ANON_NETMASK=\$( echo \$SCHOOL_ANON_DHCP_NET | gawk -F '/' '{ print \$2 }' )
echo "Setup mysql OSS database"
sed -i "s/#SERVER_NETWORK#/\${SERVER_NETWORK}/g"             /opt/oss-java/data/school-INSERT.sql
sed -i "s/#SERVER_NETMASK#/\${SERVER_NETMASK}/g"             /opt/oss-java/data/school-INSERT.sql
sed -i "s/#ANON_NETWORK#/\${ANON_NETWORK}/g"                 /opt/oss-java/data/school-INSERT.sql
sed -i "s/#ANON_NETMASK#/\${ANON_NETMASK}/g"                 /opt/oss-java/data/school-INSERT.sql
sed -i "s/#SCHOOL_NETBIOSNAME#/\${SCHOOL_NETBIOSNAME}/g"     /opt/oss-java/data/school-INSERT.sql
sed -i "s/#SCHOOL_SERVER#/\${SCHOOL_SERVER}/g"               /opt/oss-java/data/school-INSERT.sql
sed -i "s/#SCHOOL_MAILSERVER#/\${SCHOOL_MAILSERVER}/g"       /opt/oss-java/data/school-INSERT.sql
sed -i "s/#SCHOOL_PROXY#/\${SCHOOL_PROXY}/g"                 /opt/oss-java/data/school-INSERT.sql
sed -i "s/#SCHOOL_PRINTSERVER#/\${SCHOOL_PRINTSERVER}/g"     /opt/oss-java/data/school-INSERT.sql
sed -i "s/#SCHOOL_BACKUP_SERVER#/\${SCHOOL_BACKUP_SERVER}/g" /opt/oss-java/data/school-INSERT.sql
sed -i "s/#SCHOOL_NETWORK#/\${SCHOOL_NETWORK}/g"             /opt/oss-java/data/school-INSERT.sql
sed -i "s/#SCHOOL_NETMASK#/\${SCHOOL_NETMASK}/g"             /opt/oss-java/data/school-INSERT.sql
mysql < /opt/oss-java/data/oss-objects.sql
mysql OSS < /opt/oss-java/data/school-INSERT.sql
echo "grant all on OSS.* to 'claxss'\@'localhost'  identified by '$DBPW'" | mysql
sed -i s/MYSQLPWD/$DBPW/ /opt/oss-java/conf/oss-api.properties
sed -i s/SCHOOL_NETBIOSNAME/\${SCHOOL_NETBIOSNAME}/ /opt/oss-java/conf/oss-api.properties

echo "Setup profile directory"
if [ -e \$SCHOOL_HOME_BASE/profile ]; then
   mv \$SCHOOL_HOME_BASE/profile \$SCHOOL_HOME_BASE/profiles
fi
chgrp 100  "\$SCHOOL_HOME_BASE/profiles/"
chmod 1770 "\$SCHOOL_HOME_BASE/profiles"
mv /home/sysadmins/admin /home/sysadmins/administrator
chown -R root /home/sysadmins/administrator

echo "Create certifactes"
/usr/share/oss/tools/create_server_certificates.sh -N CA
/usr/share/oss/tools/create_server_certificates.sh -N admin
/usr/share/oss/tools/create_server_certificates.sh -N schoolserver

echo "Setup apache2"
sed -i 's/^APACHE_MODULES=.*/APACHE_MODULES="actions alias auth_basic authn_file authz_host authz_groupfile authz_core authz_user autoindex cgi dir env expires include log_config mime negotiation setenvif ssl socache_shmcb userdir reqtimeout php5 rewrite authn_core proxy proxy_http proxy_connect headers"/' /etc/sysconfig/apache2
sed -i 's/^APACHE_SERVER_FLAGS=.*/APACHE_SERVER_FLAGS="SSL"/' /etc/sysconfig/apache2
sed "s/#DOMAIN#/\$SCHOOL_DOMAIN/g" /usr/share/oss/setup/templates/admin_include.conf.ini > /etc/apache2/vhosts.d/admin_include.conf
sed "s/#DOMAIN#/\$SCHOOL_DOMAIN/g" /usr/share/oss/setup/templates/oss_include.conf.ini   > /etc/apache2/vhosts.d/oss_include.conf
if [ \$SCHOOL_ISGATE = "yes" ]; then
   sed -i 's/admin:443/admin:443 extip:444/' /etc/apache2/vhosts.d/admin_include.conf
   sed -i 's/ Listen 443/ Listen 443\\n            Listen 444/' /etc/apache2/listen.conf
fi
systemctl enable apache2
systemctl start  apache2

echo "Setup cups"
cp /etc/cups/cupsd.conf.in /etc/cups/cupsd.conf
sed -i 's/^SystemGroup.*/SystemGroup root lp/' /etc/cups/cups-files.conf
usermod -a -G lp root

echo "Setup vsftpd"
if [ -e /etc/xinetd.d/vsftpd ]; then
	rm -f /etc/xinetd.d/vsftpd
fi
zypper -n install vsftpd
echo "Enable some importent services"
for i in \$( cat /usr/share/oss/setup/services-to-enable )
do
    systemctl enable \$i
done

samba-tool domain classicupgrade --dbdir=$DIR/SAMBA/ --realm="$DOMAIN" $DIR/SAMBA/smb.conf
echo " - Start samba service"
systemctl enable samba.service

########################################################################
echo " - Setup samba private krbconf to kerberos conf"
cp /var/lib/samba/private/krb5.conf /etc/krb5.conf

########################################################################
echo " - Tell nsswitch to use winbind."
cp /usr/share/oss/setup/templates/nsswitch.conf /etc/nsswitch.conf

########################################################################
echo " - Use our enhanced samba.service file."
systemctl restart samba.service
cp /usr/share/oss/setup/templates/samba.service /usr/lib/systemd/system/samba.service

########################################################################
echo " - Create linked groups directory "
mkdir -p -m 755 \$SCHOOL_HOME_BASE/groups/LINKED/
if [ \$SCHOOL_NETBIOSNAME != "admin" ]; then
    samba-tool dns add localhost \$SCHOOL_DOMAIN admin   A \$SCHOOL_SERVER        -U Administrator%"\$password"
fi

EOF

##################################################
# Reading the users
print "Processing the users.\n";
my $users  = {
		administrator => {
				id   => 1,
				role => 'sysadmins',
				sn   => 'Administrator',
				givenName   => 'Main',
				birthDay   => '2018-04-02'
		},
		tteachers => {
				id   => 2,
				role => 'sysadmins',
				sn   => 'Administrator',
				givenName   => 'Main',
				birthDay   => '2018-04-02'
		},
		tstudents => {
				id   => 3,
				role => 'sysadmins',
				sn   => 'Administrator',
				givenName   => 'Main',
				birthDay   => '2018-04-02'
		},
		tadministration => {
				id   => 4,
				role => 'sysadmins',
				sn   => 'Administrator',
				givenName   => 'Main',
				birthDay   => '2018-04-02'
		},
		tworkstations => {
				id   => 5,
				role => 'sysadmins',
				sn   => 'Administrator',
				givenName   => 'Main',
				birthDay   => '2018-04-02'
		},
		cephalix => {
				id   => 6,
				role => 'internal',
				sn   => 'Administrator',
				givenName   => 'Internal',
				birthDay   => '2018-04-02'
		},
		register => {
				id   => 7,
				role => 'internal',
				sn   => 'Administrator',
				givenName   => 'Internal',
				birthDay   => '2018-04-02'
		},
		ossreader => {
				id   => 8,
				role => 'internal',
				sn   => 'Reader',
				givenName   => 'Account',
				birthDay   => '2018-04-02'
		}

	};
my $urCount = 9;
my $result = $oss->{LDAP}->search(  base   => $oss->{SYSCONFIG}->{USER_BASE},
                                  scope  => "one",
                                  filter => "objectclass=*",
                                  attrs  => [ 'uid','uniqueidentifier','sn','givenName','role','birthDay','susemailacceptaddress','susemailforwardaddress','uidNumber','homeDirectory' ]
                                );

if($result->code)
{
    $oss->ldap_error($result);
    print LOG "Error by reading the users.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by reading the users.\n");
}

#OsNexId
my $OssNextId = 4000100;
open USERS, ">:encoding(UTF-8)","$DIR/OSS-MIGRATE-TO-4-0-prepare.users";
foreach my $entry ( $result->entries )
{
    my $uid = lc($entry->get_value("uid"));
    next if( $uid eq "admin" );
    my $sn  = $entry->get_value("sn"); 
    my $givenName = $entry->get_value("givenName"); 
    my $birthDay = $entry->get_value("birthDay"); 
    my $uuid = $entry->get_value("uniqueidentifier") || ''; 
    my $role = $entry->get_value("role"); $role =~ s/,sysadmins//; 
    my $uidNumber     = $entry->get_value("uidNumber"); 
    my $homeDirectory = $entry->get_value("homeDirectory"); 
    my @suseMailAcceptAddress = $entry->get_value("suseMailAcceptAddress"); 
    my @suseMailForwardAddess = $entry->get_value("suseMailForwardAddess");
    if( !defined $users->{$uid}->{id} ) {
       $users->{$uid}->{id} = $urCount;
       $urCount++;
    }
    Encode::_utf8_on($sn);
    Encode::_utf8_on($givenName);
    $sn =~ s/'/\\'/g;
    $givenName =~ s/'/\\'/g;
    $users->{$uid}->{uid} = $uid;
    $users->{$uid}->{uuid} = $uuid;
    $users->{$uid}->{role} = $role;
    $users->{$uid}->{sn} = $sn;
    $users->{$uid}->{givenName} = $givenName;
    $users->{$uid}->{birthDay} = $birthDay;
    $users->{$uid}->{suseMailAcceptAddress} = \@suseMailAcceptAddress;
    $users->{$uid}->{suseMailForwardAddess} = \@suseMailForwardAddess;
    if( $birthDay ) {
        print SQL "INSERT INTO Users VALUES(".$users->{$uid}->{id}.",'$uid','$uuid','$role','$sn','$givenName','$birthDay',0,0,0,0,'',1);\n";
    } else {
        print SQL "INSERT INTO Users VALUES(".$users->{$uid}->{id}.",'$uid','$uuid','$role','$sn','$givenName',NOW(),0,0,0,0,'',1);\n";
    }
    foreach(@suseMailAcceptAddress) {
	/(.*)\@.*/;
	my $alias = lc($1);
	if( $alias ne lc($uid) ) {
		print SQL "INSERT INTO Aliases VALUES(NULL,".$users->{$uid}->{id}.",'$alias');\n";
	}
    }
#    foreach(@suseMailForwardAddess) {
#	print SQL "INSERT INTO Forwarders VALUES(NULL,".$users->{$uid}->{id}.",'$_');\n";
#    }
    print USERS "$uid:$sn:$givenName:$homeDirectory:$uidNumber:$OssNextId\n";
    $OssNextId++;
}
close(USERS);


##################################################
# Reading the groups
print "Processing the groups.\n";
my $groups = {  
		sysadmins => {
				id   => 1,
				desc => "Sysadmins",
				type => "primary"
			      },
		teachers  => {
				id   => 2,
				desc => "Teachers",
				type => "primary"
			      },
		students => {
				id   => 3,
				desc => "Students",
				type => "primary"
			      },
		administration => {
				id   => 4,
				desc => "Administration",
				type => "primary"
			      },
		workstations => {
				id   => 5,
				desc => "Workstations",
				type => "primary"
			      },
		templates => {
				id   => 6,
				desc => "Templates",
				type => "primary"
			      },

	     };
my $grCount = 7;
$result = $oss->{LDAP}->search(  base   => $oss->{SYSCONFIG}->{GROUP_BASE},
                                  scope  => "one",
                                  filter => "objectclass=*",
                                  attrs  => [ 'cn','groupType','member','description','susemailacceptaddress','susemailforwardaddress','gidNumber','creatorsName' ]
                                );

if($result->code)
{
    $oss->ldap_error($result);
    print LOG "Error by reading the groups.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by reading the groups.\n");
}

open GROUPS, ">:encoding(UTF-8)","$DIR/OSS-MIGRATE-TO-4-0-prepare.groups";
foreach my $entry ( $result->entries )
{
    my $cn = lc($entry->get_value("cn"));
    if( defined $users->{$cn} ) {
        print LOG "ERROR Group $cn can not be migrated. A user with the same name does exists.\n";
	$oss->{LDAP}->delete( $entry->dn() );
	next;
    }
    my $groupType   = $entry->get_value("groupType");
    my $gidNumber   = $entry->get_value("gidNumber");
    my $description = $entry->get_value("description") || $cn;
    my $creator     = $entry->get_value("creatorsName"); $creator = lc(get_name_of_dn($creator));
    my $creatorId   = 1;
    if( $creator != "administrator" && $creator != "admin" && defined $users->{$creator}->{id} ) {
        $creatorId = $users->{$creator}->{id};
    }
    my @member  = $entry->get_value("member");
    my @suseMailAcceptAddress = $entry->get_value("suseMailAcceptAddress"); 
    my @suseMailForwardAddess = $entry->get_value("suseMailForwardAddess");
    if( !defined $groups->{$cn}->{id} ) {
       $groups->{$cn}->{id} = $grCount;
       $grCount++;
    }
    Encode::_utf8_on($description);
    $groups->{$cn}->{groupType} = $groupType;
    $groups->{$cn}->{description} = $description;
    $groups->{$cn}->{member} = \@member;
    $groups->{$cn}->{suseMailAcceptAddress} = \@suseMailAcceptAddress;
    $groups->{$cn}->{suseMailForwardAddess} = \@suseMailForwardAddess;
    if( defined $users->{$cn} ) {
        print "The group '$cn' can not be imported because a user with the same uid does exists.\n";
        print LOG "The group '$cn' can not be imported because a user with the same uid does exists.\n";
	next;
    }
    print SQL "INSERT INTO Groups VALUES(".$groups->{$cn}->{id}.",'$cn','$description','$groupType',$creatorId);\n";
    my @tmp = ();
    foreach my $udn (@member) {
        my $uid = get_name_of_dn($udn);
	if( defined $users->{$uid}->{id} ) {
 	    print SQL "INSERT INTO GroupMember VALUES(".$users->{$uid}->{id}.','.$groups->{$cn}->{id}.");\n";
            push @tmp,$uid if( $uid ne "Administrator" );
	}
    }
    print CMD "samba-tool group add $cn --description '$description'\n";
    print CMD "samba-tool group addmembers $cn ".join(",",@tmp).";\n";
    print GROUPS "$cn:$gidNumber:$OssNextId\n";
    $OssNextId++;
}
close GROUPS;

##################################################
# Reading the hwconfs
print "Processing the hardware configurations.\n";
my $hwconfs = {
		Server  => { id => 1, description => "Server", deviceType => "Server" },
		Printer => { id => 2, description => "Printer", deviceType => "Printer" },
		BYOD    => { id => 3, description => "Privat Devices", deviceType => "BYOD" },
		'Win10-64-S' => { id => 4, description => "Win10 64Bit Simple Join", deviceType => "Win10-64-S" },
		'Win10-64-D' => { id => 5, description => "Win10 64Bit Domain Join", deviceType => "Win10-64-D" },
		'Win10-64-N' => { id => 6, description => "Win10 64Bit No Join", deviceType => "Win10-64-N" },
	     };
my $hwCount = 7;
$result = $oss->{LDAP}->search( base   => $oss->{SYSCONFIG}->{COMPUTERS_BASE},
                                       scope   => 'one',
                                       filter  => '(Objectclass=SchoolConfiguration)'
                          );
if($result->code)
{
    $oss->ldap_error($result);
    print LOG "Error by reading the HWConfs.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by reading the HWConfs.\n");
}

foreach my $entry ( $result->entries )
{
    my $cn = $entry->get_value("configurationKey");
    $hwconfs->{$cn}->{id} = $hwCount;
    my $description = $entry->get_value("description");
    my $name = $description;
    $name =~ s/  / /g;
    $name =~ s/ /-/g;
    $name = substr($name,0,31);
    my @values  = $entry->get_value("configurationValue");
    my $WSType  = "FatClient";
    my $part    = {};
    foreach(@values){
        my($key,$value) = split /=/,$_;
	$WSType = $value if( $key eq "WSType");
	if( $key =~ /PART_(.*)_(.*)/ ) {
	    $part->{$1}->{$2} = $value;
	}
    }
    print SQL "INSERT INTO HWConfs Values($hwCount,'$name','$description','$WSType',1);\n";
    foreach(keys %$part) {
        my $desc   = substr($part->{$_}->{DESC},0,30);
	my $JOIN   = defined $part->{$_}->{JOIN}   ? $part->{$_}->{JOIN}   : "";
	my $OS     = defined $part->{$_}->{OS}     ? $part->{$_}->{OS}     : "";
	my $FORMAT = defined $part->{$_}->{FORMAT} ? $part->{$_}->{FORMAT} : "";
	my $ITOOL  = defined $part->{$_}->{ITOOL}  ? $part->{$_}->{ITOOL}  : "";
	if( $JOIN eq "Simple" ) {
		$JOIN = "Domain";
	}
        print SQL "INSERT INTO Partitions Values(NULL,$hwCount,'$_','$desc','$OS','$JOIN','$ITOOL','$FORMAT',1);\n";
    }
    print CMD "mv /srv/itool/images/$cn /srv/itool/images/$hwCount;\n";
    $hwCount++;
}


##################################################
# Reading the rooms
print "Processing the rooms configurations.\n";
my $rooms = {
 		SERVER_NET  => { id => 1 },
 		ANON_DHCP   => { id => 2 }
	     };
my $roomCount = 3;

$result = $oss->{LDAP}->search( base   => $oss->{SYSCONFIG}->{DHCP_BASE},
                                       scope   => 'sub',
                                       filter  => '(&(Objectclass=SchoolRoom)(description=*))'
                          );
if($result->code)
{
    $oss->ldap_error($result);
    print LOG "Error by reading the rooms.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by reading the rooms.\n");
}

foreach my $entry ( $result->entries )
{
    my $description = $entry->get_value("description");
    my $dhcpRange   = $entry->get_value("dhcpRange");
    my $dhcpNetMask = $entry->get_value("dhcpNetMask");
    my $hwconf  = "NULL";
    my @values  = $entry->get_value("configurationValue");
    my $roomType = 'ComputerRoom';
    if( !defined $rooms->{$description}->{id} ) {
       $rooms->{$description}->{id} = $roomCount;
       $roomCount++;
    }
    my $x  = 6;
    my $y  = 6;
    my $xy = $oss->get_vendor_object($entry->dn(),'EXTIS','COORDINATES');
    if( defined $xy->[0] )
    {
      ( $x,$y ) = split /,/ , $xy->[0];
    }
    foreach(@values){
       my($key,$value) = split /=/,$_;
       $hwconf = $hwconfs->{$value}->{id} if( $key eq 'HW' and $value ne '-' );
       $roomType = 'AdHocAccess' if( $key eq "AdHocWlanAccess" and lc($value) eq 'yes');
    }
    if( ! $hwconf ) {
        $hwconf  = "NULL";
    }
    my ( $control, $controller, $controllers )  = $oss->get_room_control_state($entry->dn());
    my $roomControl = $controlMap->{$control};
    #####################################   id                          hwconf_id name          description    roomType   roomControl  rows places startIP   netMask     creator_id 
    print SQL "INSERT INTO Rooms VALUES(".$rooms->{$description}->{id}.",$hwconf,'$description','$description','$roomType','$roomControl',$x,$y,'$dhcpRange',$dhcpNetMask,1);\n";
    my @dhcpStatements = $entry->get_value("dhcpStatements");
    foreach( @dhcpStatements ) {
       print SQL "INSERT INTO OSSMConfig Values(NULL,'Room',".$rooms->{$description}->{id}.",'dhcpStatements','$_',1);\n";
    }
    my @dhcpOptions = $entry->get_value("dhcpOptions");
    foreach( @dhcpOptions ) {
       print SQL "INSERT INTO OSSMConfig Values(NULL,'Room',".$rooms->{$description}->{id}.",'dhcpOptions','$_',1);\n";
    }
}


##################################################
# Reading DNS entries which ar not in DHCP

$result = $oss->{LDAP}->search( base   => $oss->{SYSCONFIG}->{DNS_BASE},
                                       scope   => 'sub',
                                       filter  => '(&(zoneName='.$DOMAIN.')(!(objectClass=DHCPEntry))(!(sRVRecord=*)))'
                          );
if($result->code)
{
    $oss->ldap_error($result);
    print LOG "Error by reading the devices.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by reading the non DHCP DNS entries.\n");
}
foreach my $entry ( $result->entries )
{
	my $cn = $entry->get_value("relativeDomainName");
	my $IP = $entry->get_value("aRecord");
	next if( !defined $IP or $IP == "");
	next if( $cn eq '@' );
	print DNS "samba-tool dns add localhost ".$DOMAIN." $cn  A $IP -U register%\"\$REGPW\";\n";

}

##################################################
# Reading the workstations
print "Processing the workstations configurations.\n";

my $devices = {
	admin => { id => 1 },
	schoolserver => { id => 2 },
	proxy => { id => 3 },
	printserver => { id => 4 },
	backup => { id => 5 },
};

my $devCount = 6;

$result = $oss->{LDAP}->search( base   => $oss->{SYSCONFIG}->{DHCP_BASE},
                                       scope   => 'sub',
                                       filter  => '(&(objectClass=DHCPEntry)(!(cn=*-wlan)))'
                          );
if($result->code)
{
    $oss->ldap_error($result);
    print LOG "Error by reading the devices.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by reading the devices.\n");
}

#Read the statistics
my $statistic = {};
my $rows  = $DBH->prepare("SELECT * FROM  logonCounter");
if( defined $rows ) {
	print "Processing the workstations usage statistics.\n";
	$rows->execute;
	while( my $row = $rows->fetch() )
	{
		$statistic->{$row->[0]} = $row->[1];
	}
}
foreach my $entry ( $result->entries )
{
    my $cn     = lc($entry->get_value("cn"));
    my $room   = $rooms->{$oss->get_attribute(get_parent_dn($entry->dn()),'description')}->{id};
    my $MAC    = $entry->get_value("dhcpHWAddress"); $MAC =~ s/ethernet //;
    my @dhcpStatements = $entry->get_value("dhcpStatements");
    my $IP     = "";
    my $owner  = 'NULL';
    my $hwconf = 'NULL';
    my $WLANIP = '';
    my $WLANMAC= '';
    my $MCONFIG= '';
    my $serial    = '';
    my $inventary = '';
    my $locality   = '';
    my $serial = '';
    if( !defined $devices->{$cn}->{id} ) {
       $devices->{$cn}->{id} = $devCount;
       $devCount++;
    }
    foreach( @dhcpStatements ) {
       my($key,$value) = split / /,$_;  
       if( $key eq 'fixed-address' ) {
           $IP = $value;
       } else {
           $MCONFIG .="INSERT INTO OSSMConfig Values(NULL,'Device',".$devices->{$cn}->{id}.",'dhcpStatements','$_',1);\n";
       }
    }
    my @dhcpOptions = $entry->get_value("dhcpOptions");
    foreach( @dhcpOptions ) {
       print SQL "INSERT INTO OSSMConfig Values(NULL,'Device',".$devices->{$cn}->{id}.",'dhcpOptions','$_',1);\n";
    }
    my @values  = $entry->get_value("configurationValue");
    foreach(@values){
       my($key,$value) = split /=/,$_;
       $hwconf = $hwconfs->{$value}->{id} if( $key eq 'HW' and $value ne '-' );
       $owner  = $users->{get_name_of_dn($value)}->{id} if( $key eq 'DEVICE_OWNER' and defined $users->{get_name_of_dn($value)}->{id});
       if( $key eq 'MASTER'  and lc($value) eq 'yes' ) {
           $MCONFIG .="INSERT INTO OSSConfig Values(NULL,'Device',".$devices->{$cn}->{id}.",'MASTER','yes',1);\n";
       }
       $serial     = $value if( $key eq 'SERIALNUMBER' );
       $inventary  = $value if( $key eq 'INVENTARNUMBER' );
       $locality   = $value if( $key eq 'LOCALITY' );
    }
    my $wlanDN = $oss->get_host($cn.'-wlan');
    if( $wlanDN ) {
        my $e = $oss->get_entry($wlanDN,1);
	$WLANMAC= $e->get_value("dhcpHWAddress"); $WLANMAC =~ s/ethernet //;
	@dhcpStatements = $e->get_value("dhcpStatements");
	foreach( @dhcpStatements ) {
	       my($key,$value) = split / /,$_;  
	       $WLANIP = $value if( $key eq 'fixed-address' );
        }
    }
    my $x  = 0;
    my $y  = 0;
    my $xy = $oss->get_vendor_object($entry->dn(),'EXTIS','COORDINATES');
    if( defined $xy->[0] )
    {
      ( $x,$y ) = split /,/ , $xy->[0];
    }
    my $count = defined $statistic->{$cn} ? $statistic->{$cn} : 0;
    print SQL "INSERT INTO Devices Values(".$devices->{$cn}->{id} .",$room,$hwconf,$owner,'$cn','$IP','$WLANIP','$MAC','$WLANMAC',$x,$y,'$serial','$inventary','$locality',$count);\n";
    print DNS "samba-tool dns add localhost ".$DOMAIN." $cn  A $IP -U register%\"\$REGPW\";\n";
    if( $WLANIP ) {
        print DNS "samba-tool dns add localhost ".$DOMAIN." $cn-wlan  A $WLANIP -U register%\"\$REGPW\";\n";
    }

    if( $MCONFIG ) {
       print SQL $MCONFIG;
    }
}


#Find all software and relations.
print "Processing the softwares and relations.\n";

my $software = {};
my $softwareCount = 1;
my $categoryCount = 1;
my $licenseCount  = 1;
my $softwareVersionCount  = 1;

$result = $oss->{LDAP}->search( base   => 'o=osssoftware,'.$oss->{SYSCONFIG}->{COMPUTERS_BASE},
                                       scope   => 'one',
                                       filter  => '(Objectclass=SchoolConfiguration)'
                          );
if($result->code)
{
    $oss->ldap_error($result);
    print LOG "Error by reading the Software.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    print("Error by reading the software.\n");
} else { 

	foreach my $entry ( $result->entries )
	{
	    my $pkgDn   = $entry->dn();
	    my $oldName = $entry->get_value("configurationKey");
	    my $name    = $oss->get_config_value($pkgDn, 'pkgNewName') || $oldName;
	    my $version = $oss->get_config_value($pkgDn, 'pkgVersion')  || '0.0';
	    my $desc    = $oss->get_config_value($pkgDn, 'pkgDescription') || '';
	    my $licType = $oss->get_config_value($pkgDn, 'pkgLicenseAllocationType') || "NONE";
	    $licType = "NONE" if( $licType eq "NO_LICENSE_KEY");
	    Encode::_utf8_on($oldName);
	    Encode::_utf8_on($name);
	    Encode::_utf8_on($desc);

	    ##Ignore some software:
	    next if $name =~ /^wsusUpdate/;
	    next if $name =~ /^OssClient/;
	    next if $name =~ /^WCDV/;
	    next if $name =~ /^WCDWindowsControlDaemon/;
	    next if $name =~ /UpdateforMicrosoft/;

	    $software->{$oldName}->{id}        = $softwareCount;
	    $software->{$oldName}->{version}   = $version;
	    $software->{$oldName}->{versionID} = $softwareVersionCount;
	    print SQL "INSERT INTO Softwares Values($softwareCount,'$name','$desc',50,'N',1);\n";
	    print SQL "INSERT INTO SoftwareVersions Values($softwareVersionCount,$softwareCount,'$version','C');\n";
	    if( $licType ne 'NONE' ) {
		#License handling
		if( $licType eq 'FILE' ) {
		    $licType = 'F';
		} else {
		    $licType = 'C';
		}
		my $result2 = $oss->{LDAP}->search( base   => 'o=productkeys,'.$pkgDn,
						    scope   => 'one',
						    filter  => '(Objectclass=SchoolConfiguration)'
				      );
		foreach my $entry2 ( $result2->entries )
		{
		    my @values  = $entry2->get_value("configurationValue");
		    my $COUNT   = $oss->get_config_value($entry2->dn(),'NUMBER_OF_PIECES');
				  chomp $COUNT;
			$COUNT  =~ s/\D+//g;
		    my $PKEY    = decode_base64($oss->get_config_value($entry2->dn(),'PRODUCT_KEY'));
		    if( $licType eq 'F' ) {
			system("mkdir -p /srv/salt/packages/$oldName/");
			write_file("/srv/salt/packages/$oldName/licenses_$licenseCount",$PKEY);
			$PKEY = "licenses_$licenseCount";
		    }
		    chomp $PKEY;
		    print SQL "INSERT INTO SoftwareLicenses Values($licenseCount,$softwareCount,'$licType' ,$COUNT,'$PKEY','1');\n";
		    foreach(@values) {
			my($key,$value) = split /=/,$_;
			$value = lc($value);
			if( $key = 'USED' and defined $devices->{$value}->{id} ) {
				print SQL "INSERT INTO LicenseToDevice Values($licenseCount,".$devices->{$value}->{id}.");\n";
			}
		    }
		    $licenseCount++;
		}
	    }
	    $softwareCount++;
	    $softwareVersionCount++;
	}
	$result = $oss->{LDAP}->search(  base   => $oss->{SYSCONFIG}->{USER_BASE},
					 scope  => "sub",
					 filter => "o=osssoftware",
					 attrs  => [ 'dn' ]
					);

	if($result->code)
	{
	    $oss->ldap_error($result);
	    print LOG "Error by reading the software states on the devices.\n";
	    print LOG $oss->{ERROR}->{code};
	    print LOG $oss->{ERROR}->{text};
	    error("Error by reading the software states on the devices.\n");
	}
	my $manuellSoftware = {};
	foreach my $entry ( $result->entries )
	{
	    my $result2 = $oss->{LDAP}->search( base   => $entry->dn(),
						scope   => 'one',
						filter  => '(Objectclass=SchoolConfiguration)'
				  );
	    my $PC = lc(get_name_of_dn(get_parent_dn(get_parent_dn($entry->dn()))));
	    foreach my $entry2 ( $result2->entries )
	    {
		my $STATUS = $oss->get_config_value($entry2->dn(),'pkgStatus');
		next if(!defined $softwareStatusMap->{$STATUS});
		next if($STATUS eq 'installation_failed');
		next if($STATUS eq 'deinstallation_scheduled');
		next if($STATUS eq 'deinstallation_failed');
		next if($STATUS eq 'deinstalled_manual');

		$STATUS = $softwareStatusMap->{$STATUS};
		my $name   = get_name_of_dn($entry2->dn());
		if( defined $software->{$name} ) {
		    print SQL "INSERT INTO SoftwareStatus Values(NULL,".$software->{$name}->{versionID}.",".$devices->{$PC}->{id}.",'".$STATUS."');\n";
		} elsif( defined $manuellSoftware->{$name} ) {
		    print SQL "INSERT INTO SoftwareStatus Values(NULL,".$manuellSoftware->{$name}->{versionID}.",".$devices->{$PC}->{id}.",'".$STATUS."');\n";
		} else {
		    next if $name =~ /^wsusUpdate/;
		    next if $name =~ /^OssClient/;
		    next if $name =~ /^WCDV/;
		    next if $name =~ /^WCDWindowsControlDaemon/;
		    next if $name =~ /UpdateforMicrosoft/;
		    print SQL "INSERT INTO Softwares Values($softwareCount,'$name','',50,'Y',1);\n";
		    print SQL "INSERT INTO SoftwareVersions Values($softwareVersionCount,$softwareCount,'0.0','U');\n";
		    print SQL "INSERT INTO SoftwareStatus Values(NULL,$softwareVersionCount,".$devices->{$PC}->{id}.",'".$STATUS."');\n";
		    $manuellSoftware->{$name}->{id} = $softwareCount;
		    $manuellSoftware->{$name}->{versionID} = $softwareVersionCount;
		    $manuellSoftware->{$name}->{version} = "0.0";
		    $softwareCount++;
		    $softwareVersionCount++;
		}
	    }
	}
}
#########################################################################
# Find default printer
#
print "Processing the default and available printers.\n";
$result = $oss->{LDAP}->search( base   => $oss->{SYSCONFIG}->{DHCP_BASE},
                                       scope   => 'sub',
                                       filter  => '(&(Objectclass=SchoolRoom)(description=*))'
                          );
if($result->code)
{
    $oss->ldap_error($result);
    print LOG "Error by reading the rooms.\n";
    print LOG $oss->{ERROR}->{code};
    print LOG $oss->{ERROR}->{text};
    error("Error by reading the rooms.\n");
}

foreach my $entry ( $result->entries )
{
        my $DEFAULT_PRINTER  = $oss->get_vendor_object($entry->dn(),'EXTIS','DEFAULT_PRINTER');
        my $AVAILABLE_PRINTER= $oss->get_vendor_object($entry->dn(),'EXTIS','AVAILABLE_PRINTER');
        $DEFAULT_PRINTER     = lc($DEFAULT_PRINTER->[0]);
        $AVAILABLE_PRINTER   = lc($AVAILABLE_PRINTER->[0]);
        my $description      = $entry->get_value('description');
        my $roomId           = $rooms->{$description}->{id};
        my $printerId        = $devices->{$DEFAULT_PRINTER}->{id};
        if( defined $devices->{$DEFAULT_PRINTER}->{id} ) {
                print SQL "INSERT INTO DefaultPrinter Values(NULL,$roomId,NULL,$printerId);\n";
        }
        foreach(split(/\n/,$AVAILABLE_PRINTER)) {
                if( defined $devices->{lc($_)}->{id} ) {
                        $printerId = $devices->{lc($_)}->{id};
                        print SQL "INSERT INTO DefaultPrinter Values(NULL,$roomId,NULL,$printerId);\n";
                }
        }
}

close(LOG);
close(SQL);
print CMD <<EOF;
########################################################################
# Now we are OSS-4-0 let's show it
echo '# Copyright (c) 2012 Peter Varkoly <peter\@varkoly.de> Nürnberg, Germany.  All rights reserved.
. /etc/os-release
FQH=`hostname -f`
PS1="\$FQH:\\w # "
_bred="\$(path tput bold 2> /dev/null; path tput setaf 1 2> /dev/null)"
_sgr0="\$(path tput sgr0 2> /dev/null)"
PS1="\${PRETTY_NAME} \[\$_bred\]\$PS1\[\$_sgr0\]"
unset _bred _sgr0
' > /root/.profile

########################################################################
echo "Install and enbale salt"
zypper -n install salt salt-api salt-bash-completion salt-doc salt-master oss-proxy oss-web
systemctl enable oss-api
systemctl enable salt-master
systemctl enable oss_salt_event_watcher

########################################################################
echo " - Setup printserver "
mkdir -p /home/\$SCHOOL_WORKGROUP
/usr/share/oss/oss-migration-3-to-4/add_unix_attributes.pl \$SCHOOL_WORKGROUP
SCHOOL_DOMAIN=\$( echo "\$SCHOOL_DOMAIN" | tr "[:upper:]" "[:lower:]" )
REALM=\$( echo "\$SCHOOL_DOMAIN" | tr "[:lower:]" "[:upper:]" )
cp /usr/share/oss/setup/templates/samba-printserver.service /usr/lib/systemd/system/samba-printserver.service
cp /usr/share/oss/setup/templates/samba-printserver /etc/sysconfig/
mkdir -p /var/lib/printserver/{drivers,lock,printing,private}
mkdir -p /var/lib/printserver/drivers/{IA64,W32ALPHA,W32MIPS,W32PPC,W32X86,WIN40,x64}
chgrp -R BUILTIN\\administrators /var/lib/printserver/drivers/*
chmod -R 775 /var/lib/printserver/drivers/*
setfacl -Rdm g::rwx /var/lib/printserver/drivers/*
mkdir -p /var/log/samba/printserver/
sed    "s/#REALM#/\$REALM/g"                  /usr/share/oss/setup/templates/samba-printserver.conf.ini > /etc/samba/smb-printserver.conf
sed -i "s/#WORKGROUP#/\$SCHOOL_WORKGROUP/g"   /etc/samba/smb-printserver.conf
sed -i "s/#IPADDR#/\$SCHOOL_PRINTSERVER/g"    /etc/samba/smb-printserver.conf
net ADS JOIN -s /etc/samba/smb-printserver.conf -U Administrator%"\$password"
systemctl enable samba-printserver
#systemctl start  samba-printserver
echo " - Some additional samba settings -"
samba-tool domain passwordsettings set --max-pwd-age=365

for i in /usr/share/oss/templates/login-*.ini
do
    b=\$( basename \$i .ini )
    sed "s/#PDC-SERVER#/\${SCHOOL_NETBIOSNAME}/g" \$i > /usr/share/oss/templates/\$b
done

########################################################################
echo " - Setup our smb.conf file"
cp /etc/samba/smb.conf /etc/samba/smb.conf-orig
sed    "s/#NETBIOSNAME#/\${SCHOOL_NETBIOSNAME}/g" /usr/share/oss/setup/templates/samba-smb.conf.ini > /etc/samba/smb.conf
sed -i "s/#REALM#/\$REALM/g"                      /etc/samba/smb.conf
sed -i "s/#SCHOOL_DOMAIN#/\$SCHOOL_DOMAIN/g"      /etc/samba/smb.conf
sed -i "s/#WORKGROUP#/\$SCHOOL_WORKGROUP/g"       /etc/samba/smb.conf
sed -i "s/#GATEWAY#/\$SCHOOL_SERVER_EXT_GW/g"     /etc/samba/smb.conf
sed -i "s/#IPADDR#/\$SCHOOL_SERVER/g"             /etc/samba/smb.conf
sed -i "s#HOMEBASE#\$SCHOOL_HOME_BASE#g"          /etc/samba/smb.conf

########################################################################
echo " - Setup ntp signd directory rights."
chown root:ntp /var/lib/samba/ntp_signd/
chmod 750      /var/lib/samba/ntp_signd/

########################################################################
echo "Setup ntp"
mv /etc/ntp.conf /etc/ntp.conf.backup
cp /usr/share/oss/setup/templates/ntp.conf /etc/ntp.conf
systemctl start ntpd
systemctl enable ntpd

########################################################################
echo "Add new internal accounts"
cephalixpw=`mktemp XXXXXXXXXX`
samba-tool domain passwordsettings set --complexity=off
samba-tool user create cephalix "\$cephalixpw"
samba-tool group addmembers "Domain Admins" cephalix
sed -i s/REGISTERPW/\$cephalixpw/ /opt/oss-java/conf/oss-api.properties
samba-tool user setexpiry --noexpiry cephalix
samba-tool user create register "\$cephalixpw"
samba-tool user setpassword register --newpassword=\$cephalixpw
samba-tool user setexpiry --noexpiry register
samba-tool group addmembers "Administrators" register
samba-tool user create ossreader ossreader
samba-tool user setexpiry --noexpiry ossreader
chmod 755 $DIR/OSS-MIGRATE-TO-4-0.dns
$DIR/OSS-MIGRATE-TO-4-0.dns "\$cephalixpw"
/usr/share/oss/setup/scripts/oss-setup.sh --dhcp
mysql -f OSS < /home/archiv/migrate-to-4-0/OSS-MIGRATE-TO-4-0-prepare.sql
oss_api.sh PUT devices/refreshConfig
oss_api.sh PUT softwares/saveState
for i in \$( host admin | gawk '{ print \$4 }' )
do
	if [ \$i != \$SCHOOL_SERVER ] ; then
		samba-tool dns delete localhost $DOMAIN admin  A \$i -U register%\$cephalixpw
	fi
done

EOF

close(CMD);
close(DNS);

#Some last things
my $SCHOOL_NETMASK = $oss->get_school_config("SCHOOL_NETMASK");
my $SCHOOL_NETWORK = $oss->get_school_config("SCHOOL_NETWORK");
my $SCHOOL_WORKGROUP = uc($oss->get_school_config("SCHOOL_WORKGROUP"));
my @SCHOOL_ANON_DHCP_RANGE = split(" ",$oss->get_school_config("SCHOOL_ANON_DHCP_RANGE"));
my $SCHOOL_ANON_DHCP_START = $SCHOOL_ANON_DHCP_RANGE[0];
my $block = new Net::Netmask($SCHOOL_NETWORK.'/'.$SCHOOL_NETMASK );
$oss->set_school_config("SCHOOL_NETMASK",$block->bits());
$oss->set_school_config("SCHOOL_WORKGROUP",$SCHOOL_WORKGROUP);
$oss->add_school_config("SCHOOL_NETMASK_STRING",$SCHOOL_NETMASK,"Network Mask of the internal network.","string","yes","Basis");
$oss->add_school_config("SCHOOL_ANON_DHCP_NET","$SCHOOL_ANON_DHCP_START/24","IP-network for new and not registered clients.","string","yes","Basis");

$cmd = "chmod 755 $DIR/OSS-MIGRATE-TO-4-0.sh;
pdbedit -i ldapsam:ldap://ldap -e tdbsam:$DIR/SAMBA/passdb.tdb;\n";

#Cleanup products
$cmd .= 'oss_ldap_to_sysconfig.pl
sed -i s/Basis/Basic/ /etc/sysconfig/schoolserver
. /etc/sysconfig/schoolserver
rm /etc/products.d/sle-sdk.prod /etc/products.d/SUSE_SLES.prod /etc/products.d/baseproduct
ln -s /etc/products.d/sles-oss.prod /etc/products.d/baseproduct
rpm -e --nodeps gdm suse-build-key nscd
rpm -e --nodeps pam-32bit cryptconfig-32bit pam-modules-32bit pam_mount-32bit
cp /etc/hosts /etc/hosts-backup
sed -i s/schooladmin.$SCHOOL_DOMAIN//g /etc/hosts
sed -i s/schooladmin//g /etc/hosts
if [ -e /etc/hostname ]; then
	cp /etc/hostname /etc/hostname-backup
	sed -i s/schooladmin/admin/g /etc/hostname
fi
cp /etc/HOSTNAME /etc/HOSTNAME-backup
sed -i s/schooladmin/admin/g /etc/HOSTNAME
sed -i s/DISPLAYMANAGER=.*/DISPLAYMANAGER=\"lightdm\"/ /etc/sysconfig/displaymanager
sed -i s/DEFAULT_WM=.*/DEFAULT_WM=\"xfce\"/ /etc/sysconfig/windowmanager
mv /etc/samba /etc/samba-old
mv /var/lib/samba  /var/lib/samba-old
mkdir -p /etc/samba
mkdir -p /var/lib/samba/{drivers,lock,printing,private}
';

print LOG $cmd;
system($cmd);
################################################
# Adapt smb.conf
my $m = new Config::IniFiles( -file => "$DIR/SAMBA/smb.conf" );
$m->delval("global","ldap admin dn");
$m->delval("global","ldap group suffix");
$m->delval("global","ldap idmap suffix");
$m->delval("global","ldap machine suffix");
$m->delval("global","ldap user suffix");
$m->delval("global","ldap suffix");
$m->delval("global","ldap passwd sync");
$m->delval("global","ldap passwd sync");
$m->setval("global","passdb backend","tdbsam");
$m->setval("global","netbios name","admin");
$m->delval("global","netbios aliases");
$m->newval("global","realm",uc($DOMAIN));
$m->RewriteConfig();

